"use strict";
var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const callers = require("./callers.cjs");
const Context = require("./Context.cjs");
const ContextHelper = require("./ContextHelper.cjs");
const utils = require("./utils.cjs");
class Hookable {
  constructor() {
    __publicField(this, "_hooks");
    __publicField(this, "_before");
    __publicField(this, "_after");
    __publicField(this, "_deprecatedHooks");
    __publicField(this, "_deprecatedMessages");
    __publicField(this, "hook", (name, fn, options = {}) => {
      if (!name || typeof fn !== "function") {
        return utils.noop;
      }
      const originalName = name;
      let dep;
      while (this._deprecatedHooks[name]) {
        dep = this._deprecatedHooks[name];
        name = dep.to;
      }
      if (dep && !options.allowDeprecated) {
        let message = dep.message;
        if (!message) {
          message = `${originalName} hook has been deprecated${dep.to ? `, please use ${dep.to}` : ""}`;
        }
        if (!this._deprecatedMessages) {
          this._deprecatedMessages = /* @__PURE__ */ new Set();
        }
        if (!this._deprecatedMessages.has(message)) {
          console.warn(message);
          this._deprecatedMessages.add(message);
        }
      }
      if (!fn.name) {
        try {
          Object.defineProperty(fn, "name", {
            get: () => `_${name.replace(/\W+/g, "_")}_hook_cb`,
            configurable: true
          });
        } catch {
        }
      }
      if (!this._hooks.has(name)) {
        this._hooks.set(name, []);
      }
      const hooks = this._hooks.get(name);
      hooks.push(fn);
      return () => {
        if (fn) {
          this.removeHook(name, fn);
          fn = void 0;
        }
      };
    });
    __publicField(this, "hookOnce", (name, fn) => {
      let _un;
      let _fn = (...args) => {
        if (_un) {
          _un();
        }
        _un = void 0;
        _fn = void 0;
        return fn(...args);
      };
      _un = this.hook(name, _fn);
      return _un;
    });
    __publicField(this, "removeHook", (name, fn) => {
      const hooks = this._hooks.get(name);
      if (hooks) {
        const index = hooks.indexOf(fn);
        if (index !== -1) {
          hooks.splice(index, 1);
        }
        if (hooks.length === 0) {
          this._hooks.delete(name);
        }
      }
    });
    __publicField(this, "deprecateHook", (name, deprecated) => {
      this._deprecatedHooks[name] = typeof deprecated === "string" ? { to: deprecated } : deprecated;
      const hooks = this._hooks.get(name) || [];
      this._hooks.delete(name);
      for (const hook of hooks) {
        this.hook(name, hook);
      }
    });
    __publicField(this, "deprecateHooks", (deprecatedHooks) => {
      Object.assign(this._deprecatedHooks, deprecatedHooks);
      for (const name in deprecatedHooks) {
        this.deprecateHook(name, deprecatedHooks[name]);
      }
    });
    /**
     * 添加 hooks
     */
    __publicField(this, "addHooks", (configHooks) => {
      const hooks = utils.flatHooks(configHooks);
      let removeFns = Object.keys(hooks).map((key) => this.hook(key, hooks[key]));
      return () => {
        for (const rm of removeFns) {
          rm();
        }
        removeFns = [];
      };
    });
    __publicField(this, "removeAllHooks", () => {
      this._hooks = /* @__PURE__ */ new Map();
    });
    /**
     * 同步执行 hooks，并返回最终结果
     */
    __publicField(this, "callHookSync", (name, ...args) => {
      return this.callHookWith(callers.syncTaskCaller, name, ...args);
    });
    /**
     * 异步串行执行 hooks，并返回最终结果
     */
    __publicField(this, "callHook", (name, ...args) => {
      return this.callHookWith(callers.serialTaskCaller, name, ...args);
    });
    /**
     * 异步并行执行 hooks，并返回结果列表
     */
    __publicField(this, "callHookParallel", (name, ...args) => {
      return this.callHookWith(callers.parallelTaskCaller, name, ...args);
    });
    /**
     * 根据指定 Caller 执行 hooks
     */
    __publicField(this, "callHookWith", (caller, name, ...args) => {
      let hooks = this._hooks.get(name) || [];
      let ctx = ContextHelper.ContextHelper.create(name, args, hooks.length);
      ContextHelper.ContextHelper.set(ctx);
      callEachWith(Context.ExecutionStage.before, this._before);
      ContextHelper.ContextHelper.updateStage(ctx, Context.ExecutionStage.hook);
      const result = caller(
        name,
        hooks.map((h, i) => ContextHelper.ContextHelper.with(h, ctx, i)),
        args
      );
      if (result instanceof Promise) {
        return result.finally(() => callEachWith(Context.ExecutionStage.after, this._after));
      }
      callEachWith(Context.ExecutionStage.after, this._after);
      return result;
      function callEachWith(stage, callbacks) {
        if (callbacks) {
          ContextHelper.ContextHelper.updateStage(ctx, stage);
          const len = callbacks.length;
          for (let i = 0; i < len; i++) {
            ContextHelper.ContextHelper.updateCurrentIndex(ctx, i);
            callbacks[i](ctx);
          }
        }
        if (stage === Context.ExecutionStage.after) {
          freeMem();
        }
      }
      function freeMem() {
        Promise.resolve().finally(() => {
          caller = null;
          ctx = null;
          hooks = null;
        });
      }
    });
    __publicField(this, "beforeEach", (fn) => {
      this._before = this._before || [];
      this._before.push(fn);
      return () => {
        if (this._before) {
          const index = this._before.indexOf(fn);
          if (index !== -1) {
            this._before.splice(index, 1);
          }
        }
      };
    });
    __publicField(this, "afterEach", (fn) => {
      this._after = this._after || [];
      this._after.push(fn);
      return () => {
        if (this._after) {
          const index = this._after.indexOf(fn);
          if (index !== -1) {
            this._after.splice(index, 1);
          }
        }
      };
    });
    /**
     * 获取 Hook 执行上下文
     *
     * ***注意：仅支持在 Hook 内顶级作用域同步获取！***
     *
     * @param hookName Hook 名称，用于获取指定 hook 的类型提示，推荐传入
     *
     * @example
     * ```ts
     * hooks.hook('test', () => {
     *   const ctx = hooks.getContext('test')!
     *   // => HookContext { ... }
     * })
     * ```
     */
    // @ts-expect-error hookName 用于智能感知
    // eslint-disable-next-line unused-imports/no-unused-vars
    __publicField(this, "getContext", (hookName) => {
      return ContextHelper.ContextHelper.get();
    });
    this._hooks = /* @__PURE__ */ new Map();
    this._before = void 0;
    this._after = void 0;
    this._deprecatedMessages = void 0;
    this._deprecatedHooks = {};
    this.removeHooks = this.removeHooks.bind(this);
  }
  removeHooks(nameOrConfigHooks) {
    if (typeof nameOrConfigHooks === "string") {
      nameOrConfigHooks = [nameOrConfigHooks];
    }
    if (Array.isArray(nameOrConfigHooks)) {
      nameOrConfigHooks.forEach((name) => this._hooks.delete(name));
      return;
    }
    const hooks = utils.flatHooks(nameOrConfigHooks);
    for (const key in hooks) {
      this.removeHook(key, hooks[key]);
    }
  }
}
function createHooks() {
  return new Hookable();
}
exports.Hookable = Hookable;
exports.createHooks = createHooks;
//# sourceMappingURL=Hookable.cjs.map
