"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const utils = require("./utils.cjs");
const P = Promise.resolve();
var CallStrategy = /* @__PURE__ */ ((CallStrategy2) => {
  CallStrategy2[CallStrategy2["Sync"] = 0] = "Sync";
  CallStrategy2[CallStrategy2["AsyncSerial"] = 1] = "AsyncSerial";
  CallStrategy2[CallStrategy2["AsyncParallel"] = 2] = "AsyncParallel";
  return CallStrategy2;
})(CallStrategy || {});
const CALL_STRATEGY = {
  [
    0
    /* Sync */
  ](tasks) {
    return (...args) => tasks.reduce((_, task) => task(...args), void 0);
  },
  [
    1
    /* AsyncSerial */
  ](tasks) {
    return (...args) => tasks.reduce((p, task) => p.then(() => task(...args)), P);
  },
  [
    2
    /* AsyncParallel */
  ](tasks) {
    return (...args) => Promise.all(tasks.map((task) => task(...args)));
  }
};
function callWithTask(fn, task) {
  return (...args) => task.run(() => fn(...args));
}
function createCaller(strategy) {
  const strategyFn = CALL_STRATEGY[strategy];
  let caller = (name, hooks, args) => {
    caller = null;
    const task = utils.createTask(name);
    return strategyFn(hooks.map((h) => callWithTask(h, task)))(...args);
  };
  return caller;
}
const syncTaskCaller = createCaller(
  0
  /* Sync */
);
const serialTaskCaller = createCaller(
  1
  /* AsyncSerial */
);
const parallelTaskCaller = createCaller(
  2
  /* AsyncParallel */
);
exports.CALL_STRATEGY = CALL_STRATEGY;
exports.CallStrategy = CallStrategy;
exports.createCaller = createCaller;
exports.parallelTaskCaller = parallelTaskCaller;
exports.serialTaskCaller = serialTaskCaller;
exports.syncTaskCaller = syncTaskCaller;
//# sourceMappingURL=callers.cjs.map
