(function(global, factory) {
  typeof exports === "object" && typeof module !== "undefined" ? factory(exports) : typeof define === "function" && define.amd ? define(["exports"], factory) : (global = typeof globalThis !== "undefined" ? globalThis : global || self, factory(global.EasyHookable = {}));
})(this, function(exports2) {
  "use strict";var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);

  function flatHooks(configHooks, hooks = {}, parentName) {
    for (const key in configHooks) {
      const subHook = configHooks[key];
      const name = parentName ? `${parentName}:${key}` : key;
      if (typeof subHook === "object" && subHook != null) {
        flatHooks(subHook, hooks, name);
      } else if (typeof subHook === "function") {
        hooks[name] = subHook;
      }
    }
    return hooks;
  }
  function mergeHooks(hooks, strategy) {
    const finalHooks = {};
    for (const hook of hooks) {
      const hooks2 = flatHooks(hook);
      for (const key in hooks2) {
        if (finalHooks[key]) {
          finalHooks[key].push(hooks2[key]);
        } else {
          finalHooks[key] = [hooks2[key]];
        }
      }
    }
    for (const key in finalHooks) {
      if (finalHooks[key].length > 1) {
        const array = finalHooks[key];
        finalHooks[key] = CALL_STRATEGY[strategy](array);
      } else {
        finalHooks[key] = finalHooks[key][0];
      }
    }
    return finalHooks;
  }
  const defaultTask = { run: (fn) => fn() };
  const _createTask = () => defaultTask;
  const createTask = typeof console.createTask !== "undefined" ? console.createTask : _createTask;
  function callEachWith(callbacks, e) {
    for (const callback of callbacks) {
      callback(e);
    }
  }
  function noop() {
  }
  const P = Promise.resolve();
  var CallStrategy = /* @__PURE__ */ ((CallStrategy2) => {
    CallStrategy2[CallStrategy2["Sync"] = 0] = "Sync";
    CallStrategy2[CallStrategy2["AsyncSerial"] = 1] = "AsyncSerial";
    CallStrategy2[CallStrategy2["AsyncParallel"] = 2] = "AsyncParallel";
    return CallStrategy2;
  })(CallStrategy || {});
  const CALL_STRATEGY = {
    [
      0
      /* Sync */
    ](tasks) {
      return (...args) => tasks.reduce((_, task) => task(...args), void 0);
    },
    [
      1
      /* AsyncSerial */
    ](tasks) {
      return (...args) => tasks.reduce((p, task) => p.then(() => task(...args)), P);
    },
    [
      2
      /* AsyncParallel */
    ](tasks) {
      return (...args) => Promise.all(tasks.map((task) => task(...args)));
    }
  };
  function callWithTask(fn, task) {
    return (...args) => task.run(() => fn(...args));
  }
  function createCaller(strategy) {
    const strategyFn = CALL_STRATEGY[strategy];
    let caller = (name, hooks, args) => {
      caller = null;
      const task = createTask(name);
      return strategyFn(hooks.map((h) => callWithTask(h, task)))(...args);
    };
    return caller;
  }
  const syncTaskCaller = createCaller(
    0
    /* Sync */
  );
  const serialTaskCaller = createCaller(
    1
    /* AsyncSerial */
  );
  const parallelTaskCaller = createCaller(
    2
    /* AsyncParallel */
  );
  var ExecutionStage = /* @__PURE__ */ ((ExecutionStage2) => {
    ExecutionStage2[ExecutionStage2["init"] = 0] = "init";
    ExecutionStage2[ExecutionStage2["before"] = 1] = "before";
    ExecutionStage2[ExecutionStage2["hook"] = 2] = "hook";
    ExecutionStage2[ExecutionStage2["after"] = 3] = "after";
    return ExecutionStage2;
  })(ExecutionStage || {});
  const isBrowser = typeof window !== "undefined";
  function createDebugger(hooks, _options = {}) {
    const options = {
      inspect: isBrowser,
      group: isBrowser,
      filter: () => true,
      ..._options
    };
    const _filter = options.filter;
    const filter = typeof _filter === "string" ? (name) => name.startsWith(_filter) : _filter;
    const _tag = options.tag ? `[${options.tag}] ` : "";
    const logPrefix = (event) => _tag + event.name + "".padEnd(event._id, "\0");
    const _idCtr = {};
    const unsubscribeBefore = hooks.beforeEach((event) => {
      if (filter !== void 0 && !filter(event.name)) {
        return;
      }
      _idCtr[event.name] = _idCtr[event.name] || 0;
      event._id = _idCtr[event.name]++;
      console.time(logPrefix(event));
    });
    const unsubscribeAfter = hooks.afterEach((event) => {
      if (filter !== void 0 && !filter(event.name)) {
        return;
      }
      if (options.group) {
        console.groupCollapsed(event.name);
      }
      if (options.inspect) {
        console.timeLog(logPrefix(event), event.args);
      } else {
        console.timeEnd(logPrefix(event));
      }
      if (options.group) {
        console.groupEnd();
      }
      _idCtr[event.name]--;
    });
    return {
      /** Stop debugging and remove listeners */
      close: () => {
        unsubscribeBefore();
        unsubscribeAfter();
      }
    };
  }
  let currentContext = null;
  const _ContextHelper = class _ContextHelper {
  };
  __publicField(_ContextHelper, "create", (name, args, length) => {
    return {
      name,
      args,
      length,
      currentIndex: -1,
      stage: ExecutionStage.init
    };
  });
  __publicField(_ContextHelper, "get", () => {
    return currentContext;
  });
  __publicField(_ContextHelper, "set", (ctx) => {
    currentContext = ctx;
    return ctx;
  });
  __publicField(_ContextHelper, "drop", () => {
    currentContext = null;
  });
  __publicField(_ContextHelper, "with", (fn, ctx, index) => {
    let _fn = (...args) => {
      _fn = null;
      try {
        _ContextHelper.set(ctx);
        _ContextHelper.updateCurrentIndex(ctx, index);
        const result = fn(...args);
        if (result instanceof Promise) {
          result.then((value) => {
            ctx.returned = value;
          });
        } else {
          ctx.returned = result;
        }
        return result;
      } finally {
        _ContextHelper.drop();
      }
    };
    return _fn;
  });
  __publicField(_ContextHelper, "updateStage", (ctx, stage) => {
    Object.assign(ctx, { stage });
  });
  __publicField(_ContextHelper, "updateCurrentIndex", (ctx, index) => {
    Object.assign(ctx, {
      currentIndex: index,
      isFirst: index === 0,
      isEnd: index === ctx.length - 1
    });
  });
  let ContextHelper = _ContextHelper;
  class Hookable {
    constructor() {
      __publicField(this, "_hooks");
      __publicField(this, "_before");
      __publicField(this, "_after");
      __publicField(this, "_deprecatedHooks");
      __publicField(this, "_deprecatedMessages");
      __publicField(this, "hook", (name, fn, options = {}) => {
        if (!name || typeof fn !== "function") {
          return noop;
        }
        const originalName = name;
        let dep;
        while (this._deprecatedHooks[name]) {
          dep = this._deprecatedHooks[name];
          name = dep.to;
        }
        if (dep && !options.allowDeprecated) {
          let message = dep.message;
          if (!message) {
            message = `${originalName} hook has been deprecated${dep.to ? `, please use ${dep.to}` : ""}`;
          }
          if (!this._deprecatedMessages) {
            this._deprecatedMessages = /* @__PURE__ */ new Set();
          }
          if (!this._deprecatedMessages.has(message)) {
            console.warn(message);
            this._deprecatedMessages.add(message);
          }
        }
        if (!fn.name) {
          try {
            Object.defineProperty(fn, "name", {
              get: () => `_${name.replace(/\W+/g, "_")}_hook_cb`,
              configurable: true
            });
          } catch {
          }
        }
        if (!this._hooks.has(name)) {
          this._hooks.set(name, []);
        }
        const hooks = this._hooks.get(name);
        hooks.push(fn);
        return () => {
          if (fn) {
            this.removeHook(name, fn);
            fn = void 0;
          }
        };
      });
      __publicField(this, "hookOnce", (name, fn) => {
        let _un;
        let _fn = (...args) => {
          if (_un) {
            _un();
          }
          _un = void 0;
          _fn = void 0;
          return fn(...args);
        };
        _un = this.hook(name, _fn);
        return _un;
      });
      __publicField(this, "removeHook", (name, fn) => {
        const hooks = this._hooks.get(name);
        if (hooks) {
          const index = hooks.indexOf(fn);
          if (index !== -1) {
            hooks.splice(index, 1);
          }
          if (hooks.length === 0) {
            this._hooks.delete(name);
          }
        }
      });
      __publicField(this, "deprecateHook", (name, deprecated) => {
        this._deprecatedHooks[name] = typeof deprecated === "string" ? { to: deprecated } : deprecated;
        const hooks = this._hooks.get(name) || [];
        this._hooks.delete(name);
        for (const hook of hooks) {
          this.hook(name, hook);
        }
      });
      __publicField(this, "deprecateHooks", (deprecatedHooks) => {
        Object.assign(this._deprecatedHooks, deprecatedHooks);
        for (const name in deprecatedHooks) {
          this.deprecateHook(name, deprecatedHooks[name]);
        }
      });
      /**
       * 添加 hooks
       */
      __publicField(this, "addHooks", (configHooks) => {
        const hooks = flatHooks(configHooks);
        let removeFns = Object.keys(hooks).map((key) => this.hook(key, hooks[key]));
        return () => {
          for (const rm of removeFns) {
            rm();
          }
          removeFns = [];
        };
      });
      __publicField(this, "removeAllHooks", () => {
        this._hooks = /* @__PURE__ */ new Map();
      });
      /**
       * 同步执行 hooks，并返回最终结果
       */
      __publicField(this, "callHookSync", (name, ...args) => {
        return this.callHookWith(syncTaskCaller, name, ...args);
      });
      /**
       * 异步串行执行 hooks，并返回最终结果
       */
      __publicField(this, "callHook", (name, ...args) => {
        return this.callHookWith(serialTaskCaller, name, ...args);
      });
      /**
       * 异步并行执行 hooks，并返回结果列表
       */
      __publicField(this, "callHookParallel", (name, ...args) => {
        return this.callHookWith(parallelTaskCaller, name, ...args);
      });
      /**
       * 根据指定 Caller 执行 hooks
       */
      __publicField(this, "callHookWith", (caller, name, ...args) => {
        let hooks = this._hooks.get(name) || [];
        let ctx = ContextHelper.create(name, args, hooks.length);
        ContextHelper.set(ctx);
        callEachWith2(ExecutionStage.before, this._before);
        ContextHelper.updateStage(ctx, ExecutionStage.hook);
        const result = caller(
          name,
          hooks.map((h, i) => ContextHelper.with(h, ctx, i)),
          args
        );
        if (result instanceof Promise) {
          return result.finally(() => callEachWith2(ExecutionStage.after, this._after));
        }
        callEachWith2(ExecutionStage.after, this._after);
        return result;
        function callEachWith2(stage, callbacks) {
          if (callbacks) {
            ContextHelper.updateStage(ctx, stage);
            const len = callbacks.length;
            for (let i = 0; i < len; i++) {
              ContextHelper.updateCurrentIndex(ctx, i);
              callbacks[i](ctx);
            }
          }
          if (stage === ExecutionStage.after) {
            freeMem();
          }
        }
        function freeMem() {
          Promise.resolve().finally(() => {
            caller = null;
            ctx = null;
            hooks = null;
          });
        }
      });
      __publicField(this, "beforeEach", (fn) => {
        this._before = this._before || [];
        this._before.push(fn);
        return () => {
          if (this._before) {
            const index = this._before.indexOf(fn);
            if (index !== -1) {
              this._before.splice(index, 1);
            }
          }
        };
      });
      __publicField(this, "afterEach", (fn) => {
        this._after = this._after || [];
        this._after.push(fn);
        return () => {
          if (this._after) {
            const index = this._after.indexOf(fn);
            if (index !== -1) {
              this._after.splice(index, 1);
            }
          }
        };
      });
      /**
       * 获取 Hook 执行上下文
       *
       * ***注意：仅支持在 Hook 内顶级作用域同步获取！***
       *
       * @param hookName Hook 名称，用于获取指定 hook 的类型提示，推荐传入
       *
       * @example
       * ```ts
       * hooks.hook('test', () => {
       *   const ctx = hooks.getContext('test')!
       *   // => HookContext { ... }
       * })
       * ```
       */
      // @ts-expect-error hookName 用于智能感知
      // eslint-disable-next-line unused-imports/no-unused-vars
      __publicField(this, "getContext", (hookName) => {
        return ContextHelper.get();
      });
      this._hooks = /* @__PURE__ */ new Map();
      this._before = void 0;
      this._after = void 0;
      this._deprecatedMessages = void 0;
      this._deprecatedHooks = {};
      this.removeHooks = this.removeHooks.bind(this);
    }
    removeHooks(nameOrConfigHooks) {
      if (typeof nameOrConfigHooks === "string") {
        nameOrConfigHooks = [nameOrConfigHooks];
      }
      if (Array.isArray(nameOrConfigHooks)) {
        nameOrConfigHooks.forEach((name) => this._hooks.delete(name));
        return;
      }
      const hooks = flatHooks(nameOrConfigHooks);
      for (const key in hooks) {
        this.removeHook(key, hooks[key]);
      }
    }
  }
  function createHooks() {
    return new Hookable();
  }
  exports2.CALL_STRATEGY = CALL_STRATEGY;
  exports2.CallStrategy = CallStrategy;
  exports2.ExecutionStage = ExecutionStage;
  exports2.Hookable = Hookable;
  exports2.callEachWith = callEachWith;
  exports2.createCaller = createCaller;
  exports2.createDebugger = createDebugger;
  exports2.createHooks = createHooks;
  exports2.createTask = createTask;
  exports2.flatHooks = flatHooks;
  exports2.mergeHooks = mergeHooks;
  exports2.noop = noop;
  exports2.parallelTaskCaller = parallelTaskCaller;
  exports2.serialTaskCaller = serialTaskCaller;
  exports2.syncTaskCaller = syncTaskCaller;
  Object.defineProperty(exports2, Symbol.toStringTag, { value: "Module" });
});
//# sourceMappingURL=easy-hookable.min.js.map
