"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const callers = require("./callers.cjs");
function flatHooks(configHooks, hooks = {}, parentName) {
  for (const key in configHooks) {
    const subHook = configHooks[key];
    const name = parentName ? `${parentName}:${key}` : key;
    if (typeof subHook === "object" && subHook != null) {
      flatHooks(subHook, hooks, name);
    } else if (typeof subHook === "function") {
      hooks[name] = subHook;
    }
  }
  return hooks;
}
function mergeHooks(hooks, strategy) {
  const finalHooks = {};
  for (const hook of hooks) {
    const hooks2 = flatHooks(hook);
    for (const key in hooks2) {
      if (finalHooks[key]) {
        finalHooks[key].push(hooks2[key]);
      } else {
        finalHooks[key] = [hooks2[key]];
      }
    }
  }
  for (const key in finalHooks) {
    if (finalHooks[key].length > 1) {
      const array = finalHooks[key];
      finalHooks[key] = callers.CALL_STRATEGY[strategy](array);
    } else {
      finalHooks[key] = finalHooks[key][0];
    }
  }
  return finalHooks;
}
const defaultTask = { run: (fn) => fn() };
const _createTask = () => defaultTask;
const createTask = typeof console.createTask !== "undefined" ? console.createTask : _createTask;
function callEachWith(callbacks, e) {
  for (const callback of callbacks) {
    callback(e);
  }
}
function noop() {
}
exports.callEachWith = callEachWith;
exports.createTask = createTask;
exports.flatHooks = flatHooks;
exports.mergeHooks = mergeHooks;
exports.noop = noop;
//# sourceMappingURL=utils.cjs.map
