import { AnyFn } from '@rhao/types-base';
import { NestedHooks } from './types';
import { CallStrategy } from './callers';
/**
 * 扁平化嵌套的 hooks
 * @param configHooks 嵌套的 hooks
 * @param hooks 扁平化后的 hooks
 * @param parentName 父级名称
 *
 * @example
 * ```ts
 * flatHooks({
 *   a: {
 *     b: {
 *       c: () => {}
 *     },
 *     e: () => {}
 *   }
 * })
 * // => { 'a:b:c': () => {}, 'a:e': () => {} }
 * ```
 */
export declare function flatHooks<T>(configHooks: NestedHooks<T>, hooks?: T, parentName?: string): T;
/**
 * 合并 hooks
 * @param hooks hooks 配置集合
 * @param strategy 执行策略
 *
 * @example
 * ```ts
 * mergeHooks(
 *   [{ a: () => console.log(1) }, { a: () => console.log(2) }],
 *   CallStrategy.Sync
 * )
 * // => { a: () => {...} }
 * ```
 */
export declare function mergeHooks<T>(hooks: NestedHooks<T>[], strategy: CallStrategy): T;
export declare const createTask: (name: string) => {
    run: (fn: AnyFn) => void;
};
export declare function callEachWith(callbacks: AnyFn[], e?: any): void;
export declare function noop(): void;
